#include <windows>
#pragma option push -a1
   #include <setupapi>
#pragma option pop
#include <assert>
#include <iostream>

using namespace std;

void displayError(const char* msg){
  cout << msg << endl;
  system("PAUSE");
  exit(0);
};
//---------------------------------------------------------
template <class T>
inline void releaseMemory(T &x)
{
  assert(x != NULL);
  delete [] x;
  x = NULL;
}
//---------------------------------------------------------
typedef USHORT USAGE, *PUSAGE;
typedef struct _HIDP_PREPARSED_DATA *PHIDP_PREPARSED_DATA;
//--------------------------------------------------------
typedef struct _HIDP_CAPS {
   USAGE Usage;
   USAGE UsagePage;
   USHORT InputReportByteLength;
   USHORT OutputReportByteLength;
   USHORT FeatureReportByteLength;
   USHORT Reserved[17];
   USHORT NumberLinkCollectionNodes;
   USHORT NumberInputButtonCaps;
   USHORT NumberInputValueCaps;
   USHORT NumberInputDataIndices;
   USHORT NumberOutputButtonCaps;
   USHORT NumberOutputValueCaps;
   USHORT NumberOutputDataIndices;
   USHORT NumberFeatureButtonCaps;
   USHORT NumberFeatureValueCaps;
   USHORT NumberFeatureDataIndices;
} HIDP_CAPS, *PHIDP_CAPS;
//---------------------------------------------------------
typedef struct _DEVICE_DATA {
  TCHAR   *HardwareId;
  TCHAR   *Path;
  DWORD   DeviceInstance;
  HANDLE  hidDeviceObject;
  BYTE    *inputReportBuffer;
  USHORT  inputReportByteLength;
  BYTE    *outputReportBuffer;
  USHORT  outputReportByteLength;
  BYTE    *featureReportBuffer;
  USHORT  featureReportByteLength;
  /*...*/
  PHIDP_PREPARSED_DATA preparsedData;
  HIDP_CAPS capabilities;
} DEVICE_DATA, *PDEVICE_DATA;
//---------------------------------------------------------
PDEVICE_DATA deviceList;
//---------------------------------------------------------
BOOL getHidDeviceCapabilities(UINT memberIndex)
{
   HMODULE hHidLib;
   bool status;

   long (__stdcall* HidP_GetCaps)(IN PHIDP_PREPARSED_DATA PreparsedData,
                                 OUT PHIDP_CAPS Capabilities);
   bool (__stdcall* HidD_GetPreparsedData)(IN HANDLE  HidDeviceObject,
                                    OUT PHIDP_PREPARSED_DATA *PreparsedData);

   bool (__stdcall* HidD_FreePreparsedData)
                                    (IN PHIDP_PREPARSED_DATA PreparsedData);

   hHidLib = LoadLibrary("C:\\Windows\\system32\\HID.DLL");
   if (!hHidLib)
      displayError("Bd doczenia biblioteki HID.DLL.");
  
     (FARPROC&) HidP_GetCaps=GetProcAddress(hHidLib,
                                             "HidP_GetCaps");
     (FARPROC&) HidD_GetPreparsedData=GetProcAddress(hHidLib,
                                             "HidD_GetPreparsedData");
     (FARPROC&) HidD_FreePreparsedData=GetProcAddress(hHidLib,
                                             "HidD_FreePreparsedData");

   if (!HidP_GetCaps || !HidD_GetPreparsedData || !HidD_FreePreparsedData){
      FreeLibrary(hHidLib);
      displayError("Nie znaleziono jednej lub wicej funkcji eksportowych.\n");
   }
   status=HidD_GetPreparsedData(deviceList[memberIndex].hidDeviceObject,
                                &deviceList->preparsedData);
   if(status){
      HidP_GetCaps(deviceList->preparsedData, &deviceList->capabilities);
      deviceList[memberIndex].inputReportByteLength =
                               deviceList->capabilities.InputReportByteLength;
      deviceList[memberIndex].outputReportByteLength =
                               deviceList->capabilities.OutputReportByteLength;
      deviceList[memberIndex].featureReportByteLength =
                               deviceList->capabilities.FeatureReportByteLength;
      HidD_FreePreparsedData(deviceList->preparsedData);
   }
   FreeLibrary(hHidLib);
   return status;
}
//---------------------------------------------------------
UINT setGetHidDeviceData()
{
    DWORD propertyBufferSize = 0;
    char *propertyBuffer = NULL;
    HMODULE hHidLib;
    SP_DEVINFO_DATA deviceInfoData;
    HDEVINFO deviceInfoSet;
    SP_INTERFACE_DEVICE_DATA deviceInterfaceData;
    DWORD  memberIndex = 0;
    GUID classGuid;
    PSP_DEVICE_INTERFACE_DETAIL_DATA deviceInterfaceDetailData = NULL;
    DWORD requiredSize = 0;
    DWORD deviceInterfaceDetailDataSize = 0;
    DWORD searchMaxDevice = 100; //maksymalna liczba interfejsw urzdze
    bool done = false;

    void (__stdcall *HidD_GetHidGuid)(OUT LPGUID HidGuid);

    hHidLib = LoadLibrary("C:\\Windows\\system32\\HID.DLL");
    if (!hHidLib)
      displayError("Bd doczenia biblioteki HID.DLL.");

    (FARPROC&) HidD_GetHidGuid = GetProcAddress(hHidLib,
                                              "HidD_GetHidGuid");
    if (!HidD_GetHidGuid){
       FreeLibrary(hHidLib);
       displayError("Nie znaleziono identyfikatora GUID.");
    }

    HidD_GetHidGuid (&classGuid);

    deviceInfoSet = SetupDiGetClassDevs(&classGuid, NULL, NULL,
                                       (DIGCF_PRESENT | DIGCF_DEVICEINTERFACE));

    deviceInterfaceData.cbSize = sizeof(SP_INTERFACE_DEVICE_DATA);

    while(!done) {
       deviceList =  new \
                     DEVICE_DATA[((memberIndex+1)*sizeof(DEVICE_DATA))];

       for(; memberIndex < searchMaxDevice; memberIndex++) {
          if(SetupDiEnumDeviceInterfaces(deviceInfoSet,0,&classGuid,
                                         memberIndex,&deviceInterfaceData)) {
            SetupDiGetDeviceInterfaceDetail(deviceInfoSet,&deviceInterfaceData,
                                         NULL,0,&deviceInterfaceDetailDataSize,
                                         NULL);
            requiredSize = deviceInterfaceDetailDataSize;
            deviceInterfaceDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)\
                                new DWORD[deviceInterfaceDetailDataSize];

               if(deviceInterfaceDetailData) {
                  deviceInterfaceDetailData->cbSize=\
                               sizeof(SP_INTERFACE_DEVICE_DETAIL_DATA);
                }
                else {
                   SetupDiDestroyDeviceInfoList(deviceInfoSet);
                   releaseMemory(deviceInterfaceDetailData);
                   return 0;
                }
                deviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
                if (!SetupDiGetDeviceInterfaceDetail(deviceInfoSet,
                           &deviceInterfaceData, deviceInterfaceDetailData,
                           deviceInterfaceDetailDataSize,
                           &requiredSize, &deviceInfoData)) {
                   SetupDiDestroyDeviceInfoList(deviceInfoSet);
                   releaseMemory(deviceInterfaceDetailData);
                   return 0;
                }

            size_t nLen = strlen(deviceInterfaceDetailData->DevicePath) + 1;
            deviceList[memberIndex].Path = new TCHAR[(nLen*sizeof(TCHAR))];
            strncpy(deviceList[memberIndex].Path,
                                deviceInterfaceDetailData->DevicePath, nLen);

            cout <<"\nDeviceList["<<memberIndex<<"].Path: \n" <<
                    deviceList[memberIndex].Path << endl;

            deviceList[memberIndex].DeviceInstance = deviceInfoData.DevInst;

            SetupDiGetDeviceRegistryProperty(deviceInfoSet, &deviceInfoData,
                                        SPDRP_HARDWAREID, NULL, NULL, 0,
                                        &propertyBufferSize);

            //alokowanie pamici dla bufora danych
            propertyBuffer = new char[(propertyBufferSize*sizeof(TCHAR))];

            SetupDiGetDeviceRegistryProperty(deviceInfoSet, &deviceInfoData,
                                           SPDRP_HARDWAREID,NULL,
                                           propertyBuffer, propertyBufferSize,
                                           NULL);

            deviceList[memberIndex].HardwareId  = propertyBuffer;
            cout <<"\nDeviceList["<<memberIndex<<"].HardwareId: \n" <<
                     deviceList[memberIndex].HardwareId << endl;

            }
            else {
                if(ERROR_NO_MORE_ITEMS == GetLastError()){
                  done = TRUE;
                  break;
                }
            }
            releaseMemory(deviceInterfaceDetailData);
            releaseMemory(propertyBuffer);
            //releaseMemory(deviceList[memberIndex].Path);
       }
       //releaseMemory(deviceList);
    }
    SetupDiDestroyDeviceInfoList(deviceInfoSet);
    FreeLibrary(hHidLib);
    return memberIndex;
}
//---------------------------------------------------------
HANDLE openHidUSBDevice(UINT memberIndex)
{
   deviceList[memberIndex].hidDeviceObject == INVALID_HANDLE_VALUE;
   deviceList[memberIndex].hidDeviceObject =
                      CreateFile(deviceList[memberIndex].Path,
                                 GENERIC_READ | GENERIC_WRITE,
                                 FILE_SHARE_READ | FILE_SHARE_WRITE,
                                 NULL,OPEN_EXISTING,FILE_FLAG_OVERLAPPED,NULL);

   if(deviceList[memberIndex].hidDeviceObject != INVALID_HANDLE_VALUE)
     return deviceList[memberIndex].hidDeviceObject;
     else
       return INVALID_HANDLE_VALUE;
}
//---------------------------------------------------------
BOOL closeHidUSBDevice(HANDLE devHandle)
{
  if((devHandle == 0) ||
    (devHandle == INVALID_HANDLE_VALUE)){
    return false;
  }
  else
    return CloseHandle(devHandle);
}
//---------------------------------------------------------
BOOL readUSBReport(UINT memberIndex)
{
   DWORD result = 0;
   DWORD numberOfBytesRead = 0;
   OVERLAPPED *overlapped = NULL;
   if(overlapped == NULL){
       overlapped = new OVERLAPPED;
       overlapped->hEvent = CreateEvent(NULL, TRUE, TRUE, "");
       overlapped->Offset = 0;
       overlapped->OffsetHigh = 0;
   }
   if(!ReadFile(deviceList[memberIndex].hidDeviceObject,
                deviceList[memberIndex].inputReportBuffer,
                deviceList[memberIndex].inputReportByteLength,
                &numberOfBytesRead, overlapped)) {
      if(GetLastError() == ERROR_IO_PENDING) {
          result = WaitForSingleObject(overlapped->hEvent, 100);
        if(result == WAIT_TIMEOUT) {
            CancelIo(deviceList[memberIndex].hidDeviceObject);
            return false;
        }
        else
                   if(result == WAIT_FAILED){
              displayError("Bd odczytu danych.");
              return false;
            }
        GetOverlappedResult(deviceList[memberIndex].hidDeviceObject,
                                    overlapped, &numberOfBytesRead, FALSE);
      }
      else
         displayError("Bd odczytu danych.");
   }
   ResetEvent(overlapped->hEvent);
   if(numberOfBytesRead == deviceList[memberIndex].inputReportByteLength){
     for(USHORT i=0; i< deviceList[memberIndex].inputReportByteLength; i++)
        printf("%d ", deviceList[memberIndex].inputReportBuffer[i]);
        printf("\n");
   }
   else {
     printf("Bdna liczba bajtw odebranych.\n",numberOfBytesRead);
   }
   delete overlapped;
   return true;
}
//---------------------------------------------------------
int main(){
   UINT interfaceIndex;//numer interfejsu urzdzenia HID

   //enumeracja aktualnie podczonych urzdze HID
   setGetHidDeviceData();

   cout <<"\n\n\Podaj nr istniejacego interfejsu urzadzenia = ";
   cin >> interfaceIndex;

   cout <<"\nDeviceList["<<interfaceIndex<<"].Path: \n" <<
                    deviceList[interfaceIndex].Path << endl;

   //otwarcie portu USB
   openHidUSBDevice(interfaceIndex);

   //odczyt waciwoci urzdzenia
   if (getHidDeviceCapabilities(interfaceIndex)) {

      //alokacja pamici dla bufora wejciowego
      deviceList[interfaceIndex].inputReportBuffer = \
                 new BYTE[deviceList[interfaceIndex].inputReportByteLength];

      while(true) { //cykliczny odczyt raportw z wybranego urzdzenia

        readUSBReport(interfaceIndex);

        if(deviceList[interfaceIndex].inputReportBuffer[6]==64) {
           closeHidUSBDevice(deviceList[interfaceIndex].hidDeviceObject);
           break;
        }
      }
      //zwolnienie pamici dla bufora wejciowego
      releaseMemory(deviceList[interfaceIndex].inputReportBuffer);
   }
   //zwolnienie pamici dla elementw struktury DEVICE_DATA
   //releaseMemory(deviceList[interfaceIndex].Path);
   releaseMemory(deviceList);
   cout << endl;
   system("PAUSE");
   return 0;
}
//---------------------------------------------------------
 
